import os
import automationserver.server as aut

import logging
import sys
import argparse

import fieldagent.common as fa_common

def main():

    root = logging.getLogger()
    root.setLevel(logging.DEBUG)
    ch = logging.StreamHandler(sys.stdout)
    ch.setLevel(logging.INFO)
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    ch.setFormatter(formatter)
    root.addHandler(ch)
    root.info('Start automation server ...')

    parser = argparse.ArgumentParser()
    parser.add_argument("-H", "--host", help="Automation server host of field agent.", type=str, default='localhost')
    parser.add_argument("-V", "--visa", help="PyVISA visa backend.", type=str, default='@py')
    parser.add_argument("-e", "--exchange", help="RabbitMQ exchange for RPC.", type=str, default='')
    parser.add_argument("-u", "--username", help="RabbitMQ username.", type=str, default='guest')
    parser.add_argument("-p", "--password", help="RabbitMQ password.", type=str, default='guest')
    parser.add_argument("-v", "--verbose", help="Increase output verbosity.", action="store_true")
    args = parser.parse_args()

    # Setup logging
    if args.verbose:
        root = fa_common.setupLogging(logging.DEBUG)
    else:
        root = fa_common.setupLogging(logging.INFO)

    server = aut.VisaRpcServer(
        host=args.host,
        username=args.username,
        password=args.password,
        visa_library=args.visa,
        exchange=args.exchange)

    try:
        server.start()
    except KeyboardInterrupt:
        logging.info('Closed by user.')

    server.stop()


import logging
import json

import fieldagent.rpc as rpc
import fieldagent.devicemanager as visa

class VisaRpcServer(rpc.RpcServer):
    def __init__(self, visa_library, username, password, name='visa_rpc_server', exchange="visa_rpc",
                 host='localhost'):
        rpc.RpcServer.__init__(self, name=name, host=host, exchange=exchange, username=username, password=password)
        self.device_manager = visa.VisaDeviceManager(visa_library=visa_library)

        self.create_method(name='alias', request_handler_method=self.on_alias_request)
        self.create_method(name='query', request_handler_method=self.on_query_request)
        self.create_method(name='read', request_handler_method=self.on_read_request)
        self.create_method(name='write', request_handler_method=self.on_write_request)
        self.create_method(name='list', request_handler_method=self.on_list_request)

    def on_alias_request(self, ch, method, props, body):
        request = json.loads(body)

        try:
            response = self.device_manager.alias(request)
            logging.info('LOCAL.alias: ' + json.dumps(request) + ' -> ' + json.dumps(response))
            # return response
        except Exception as e:
            logging.error(e.message)
            response = {
                'message': e.message,
                'success': False
            }

        self.return_acknowledgement(ch=ch, method=method, props=props, response=response)

    def on_query_request(self, ch, method, props, body):
        request = json.loads(body)

        try:
            response = self.device_manager.query(request)
            logging.info('LOCAL.query: ' + json.dumps(request) + ' -> ' + json.dumps(response))
        except Exception as e:
            logging.error(e.message)
            response = {
                'message': e.message,
                'success': False
            }

        self.return_acknowledgement(ch=ch, method=method, props=props, response=response)

    def on_write_request(self, ch, method, props, body):
        request = json.loads(body)

        try:
            response = self.device_manager.write(request)
            logging.info('LOCAL.write: ' + json.dumps(request) + ' -> ' + json.dumps(response))
        except Exception as e:
            logging.error(e.message)
            response = {
                'message': e.message,
                'success': False
            }

        self.return_acknowledgement(ch=ch, method=method, props=props, response=response)

    def on_read_request(self, ch, method, props, body):
        request = json.loads(body)

        try:
            response = self.device_manager.read(request)
            logging.info('LOCAL.read: ' + json.dumps(request) + ' -> ' + json.dumps(response))
        except Exception as e:
            logging.error(e.message)
            response = {
                'message': e.message,
                'success': False
            }

        self.return_acknowledgement(ch=ch, method=method, props=props, response=response)

    def on_list_request(self, ch, method, props, body):
        request = json.loads(body)

        try:
            response = self.device_manager.list(request)
            logging.info('LOCAL.list: ' + json.dumps(request) + ' -> ' + json.dumps(response))
        except Exception as e:
            logging.error(e.message)
            response = {
                'message': e.message,
                'success': False
            }

        self.return_acknowledgement(ch=ch, method=method, props=props, response=response)


#!/usr/bin/env python
import pika
import threading
import logging
import json
import uuid


class RpcError(Exception):
    def __init__(self, message):
        self.message = message


class RpcClient(object):
    def __init__(self, name, username, password, exchange='', host='localhost'):
        credentials = pika.PlainCredentials(username=username, password=password)
        self.connection = pika.BlockingConnection(pika.ConnectionParameters(host=host,credentials=credentials))
        self.channel = self.connection.channel()
        self.name = name
        self.exchangeName = exchange

    def on_response(self, ch, method, props, body):
        logging.info('on_response = ' + str(props))
        if self.corr_id == props.correlation_id:
            self.response = body

    def declare_method(self, callbackMethod):
        result = self.channel.queue_declare(exclusive=True)
        self.channel.queue_bind(exchange=self.exchangeName, queue=result.method.queue)
        self.callback_queue = result.method.queue
        logging.info('set callback queue ' + self.callback_queue)
        self.channel.basic_consume(
            self.on_response,
            no_ack=True,
            queue=self.callback_queue)

    def call(self, body, method_name):
        self.response = None
        self.corr_id = str(uuid.uuid4())
        message_properties = pika.BasicProperties(reply_to=self.callback_queue, correlation_id=self.corr_id, )
        self.channel.basic_publish(
            exchange=self.exchangeName,
            routing_key=method_name,
            properties=message_properties,
            body=json.dumps(body))
        logging.debug('Call \"' + method_name + '\" request ' + str(body) + ' -> ' + str(message_properties))
        while self.response is None:
            self.connection.process_data_events()
        logging.debug('Call \"' + method_name + '\" response ' + str(self.response) + '.')
        return json.loads(self.response)

    def dispose(self):
        self.connection.close()


class RpcServer(threading.Thread):
    def __init__(self, name, username, password, exchange='', host='localhost'):
        threading.Thread.__init__(self, name=name)
        logging.debug('name=' + str(name) + ', username=' + str(username) + ', password=' + str(password) + ', exchange=' + str(exchange) + ', host=' + str(host))
        credentials = pika.PlainCredentials(username=username, password=password)
        self.connection = pika.BlockingConnection(pika.URLParameters("amqp://tardozzi:qwe123@localhost"))
        # self.connection = pika.BlockingConnection(pika.ConnectionParameters(host=host, credentials=credentials))
        self.channel = self.connection.channel()
        self.channel.basic_qos(prefetch_count=1)
        self.exchange = exchange
        self._stop = threading.Event()
        self.channel.exchange_declare(exchange=self.exchange, type='direct')

    def stop(self):
        self._stop.set()

    def stopped(self):
        return self._stop.isSet()

    def create_method(self, name, request_handler_method):
        self.channel.queue_declare(queue=name)
        self.channel.queue_bind(exchange=self.exchange, queue=name, routing_key=name)
        logging.debug('Declare ' + self.exchange +'.' + name + '.')
        self.channel.basic_consume(request_handler_method, queue=name)

    def return_acknowledgement(self, ch, method, props, response):
        logging.debug('Acknowledge ' + str(method) + ' -> ' + str(props))
        ch.basic_publish(
            exchange=self.exchange,
            routing_key=props.reply_to,
            properties=pika.BasicProperties(correlation_id=props.correlation_id, content_type='application/json'),
            body=json.dumps(response)
        )
        ch.basic_ack(delivery_tag=method.delivery_tag)

    def dispose(self):
        self.connection.close()
        self.channel.stop_consuming()

    def run(self):
        logging.info('%s is running' % self.name)
        self.channel.start_consuming()
        self.dispose()
        return



import logging

import pika

import pyvisa as visa
import logging


class VisaDeviceManager():
    def __init__(self, visa_library):
        self.rm = visa.ResourceManager(visa_library)
        try:
            logging.info(self.rm.list_resources())
            self.devices = {}
        except Exception as ex:
            logging.exception("message")

    def alias(self, request):
        for key, value in request.iteritems():
            try:
                self.devices[key] = self.rm.open_resource(value, read_termination='\n')
            except Exception as ex:
                logging.exception("message")
        return {
            'connectedDevices': len(request)
        }

    def query(self, request):
        try:
            command = request['command']
            resource = self.devices[request['alias']]
            response = {
                'result': resource.query(command)
            }
            return response
        except Exception as ex:
            logging.exception("message")

    def read(self, request):
        try:
            command = request['command']
            resource = self.devices[request['alias']]
            response = {
                'result': resource.read(command)
            }
            return response
        except Exception as ex:
            logging.exception("message")

    def write(self, request):
        try:
            command = request['command']
            resource = self.devices[request['alias']]
            resource.write(command)
        except Exception as ex:
            logging.exception("message")

    def list(self, request):
        try:
            return self.rm.list_resources()
        except Exception as ex:
            logging.exception("message")



from setuptools import setup

def readme():
    with open('README.rst') as f:
        return f.read()

setup(name='hedgehog-fa',
      version='0.12',
      description='Stationcontroller services.',
      long_description=readme(),
      classifiers=[
            'Development Status :: 3 - Alpha',
            'License :: OSI Approved :: MIT License',
            'Programming Language :: Python :: 2.7',
            'Topic :: Text Processing :: Linguistic',
      ],
      keywords='funniest joke comedy flying circus',
      url='http://github.com/lordoftheflies/hedgehog-fa',
      author='lordoftheflies',
      author_email='heglas11@gmail.com',
      license='MIT',
      packages=['fieldagent', 'automationserver', 'cli'],
      install_requires=[
            'pika',
            'pyvisa',
      ],
      entry_points = {
          'console_scripts': [
                'fa-automationserver=automationserver.main:main',
                'fa-cli-query=cli.main:main'
          ]
      },
      zip_safe=False)




import logging
import sys

def setupLogging(logging_level):
    root = logging.getLogger()
    root.setLevel(logging_level)

    ch = logging.StreamHandler(sys.stdout)
    ch.setLevel(logging_level)
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    ch.setFormatter(formatter)
    root.addHandler(ch)
    return root;